#!/usr/bin/env python

# * ----------------------------------------------------------------------------
# * "THE BEER-WARE LICENSE" (Revision 42):
# * <thomas.langewouters@skynet.be> wrote this file. As long as you retain this notice you
# * can do whatever you want with this stuff. If we meet some day, and you think
# * this stuff is worth it, you can buy me a beer in return.
# * Thomas Langewouters
# * ---------------------------------------------------------------------------


import cgi
import cgitb; cgitb.enable()
from datetime import *
from uurrooster import Uurrooster
import socket

# A UTC class.
class UTC(tzinfo):
    """UTC"""

    def utcoffset(self, dt):
        return timedelta(0)

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return timedelta(0)

	
class ICalEvent:
	"""
	Properties:
		organizer
		organizermail
		reqpart {name: email}
		uid		unieke ID
		dtstamp		tijd waarop het event gepland werd
		dtstart		begintijd (datetime obj)
		dtend		eindtijd (datetime obj)
		summary
		location
		description
	"""
	def __init__(self):
		self.utc = UTC()
		self.host = socket.getfqdn()

	def put(self):
		self.dtstamp = self.dtstamp.astimezone(self.utc).strftime("%Y%m%dT%H%M%SZ")
		self.dtstart = self.dtstart.astimezone(self.utc).strftime("%Y%m%dT%H%M%SZ")
		self.dtend = self.dtend.astimezone(self.utc).strftime("%Y%m%dT%H%M%SZ")
		
		output = "BEGIN:VEVENT\n"
		output += "UID:%s@%s\n" % (hash((self.dtstart, self.dtend, self.description, self.summary, self.location)), self.host)
		output += "ORGANIZER;CN=%s:MAILTO:%s\n" % (self.organizer, self.organizermail)
		for name, email in self.reqpart.iteritems():
			output += "ATTENDEE;ROLE=REQ-PARTICIPANT;CN=%s:MAILTO:%s\n" % (name, email)
		# The property indicates the date/time that the instance of the iCalendar object was created.
		output += "DTSTAMP:%s\n" % self.dtstamp
		output += "DTSTART:%s\n" % self.dtstart 
		output += "DTEND:%s\n" % self.dtend
		# Event word niet herhaald
		output += "SEQUENCE:0\n"
		output += "TRANSP:OPAQUE\n"
		# Samenvatting en Lokatie...
		#output += "SUMMARY;LANGUAGE=nl;ENCODING=QUOTED-PRINTABLE:%s\n" % self.summary 
		output += "SUMMARY:%s\n" % self.summary 
		output += "LOCATION:%s\n" % self.location
		#output += "DESCRIPTION;LANGUAGE=nl;ENCODING=QUOTED-PRINTABLE:%s\n" % self.description 
		output += "DESCRIPTION:%s\n" % self.description 
		#output += "CATEGORIES;LANGUAGE=nl;ENCODING=QUOTED-PRINTABLE:Les\n";
		output += "CATEGORIES:Les\n";
		output += "CLASS:PUBLIC\n";
		output += "STATUS:CONFIRMED\n";
		output += "END:VEVENT\n";
		return output

def uurrooster2ical(uurrooster):
	calitem = ICalEvent()
	output = ""
	for les in uurrooster.lessen:
		#organizer
		calitem.organizer = les['docenten'][0]
		#organizermail
		calitem.organizermail = les['docenten'][0] + "@docent.denayer.wenk.be"
		
		#reqpart {name: email}
		rp = {}
		for nummer in les['groepen']:
			klas_groep = str(uurrooster.jaar) + str(uurrooster.richting) + str(nummer)
			rp[klas_groep] = klas_groep + "@denayer.wenk.be"
		calitem.reqpart = rp
		#uid		unieke ID
		#?
		#dtstamp		tijd waarop het event gepland werd
		calitem.dtstamp = datetime.now(les['begin'].tzinfo)
		#dtstart		begintijd (datetime obj)
		calitem.dtstart = les['begin']
		#dtend		eindtijd (datetime obj)
		calitem.dtend = les['einde'] 
		#summary
		calitem.summary = les['naam']
		#location
		calitem.location = les['lokaal']
		#description
		calitem.description = les["type"] + " door " + calitem.organizer
		output += calitem.put()
	return output

klas = cgi.FieldStorage().getfirst("klas")
mijnklas = Uurrooster(klas)
mijnklas.retrieve()
html = uurrooster2ical(mijnklas)
print "Content-type: text/calendar"
print 
print "BEGIN:VCALENDAR"
print "PRODID:-//TL//Denayerical//EN"
print "VERSION:2.0"
print"METHOD:REQUEST"
print html
print "END:VCALENDAR"
