#!/bin/env python
# * ----------------------------------------------------------------------------
# * "THE BEER-WARE LICENSE" (Revision 42):
# * <thomas.langewouters@skynet.be> wrote this file. As long as you retain this notice you
# * can do whatever you want with this stuff. If we meet some day, and you think
# * this stuff is worth it, you can buy me a beer in return.
# * Thomas Langewouters
# * ---------------------------------------------------------------------------

import urllib, re
from datetime import *

# Tijdzone class gebruikt voor een datetime object met tijdzone informatie

def first_sunday_on_or_after(dt):
    days_to_go = 6 - dt.weekday()
    if days_to_go:
        dt += timedelta(days_to_go)
    return dt

class CEST(tzinfo):
    """
    Central European Timezone (Brussels)
    forward one hour on the last Sunday in March
    back one hour on the last Sunday in October
    Central European (UTC+1)
    at 01:00 UTC
    by Thomas Langewouters.
    """

    def __init__(self):
        self.stdoffset = timedelta(hours=1)
        self.reprname = "Central European Standard Time"
        self.stdname = "CEST"
        self.dstname = "CEST"

    def __repr__(self):
        return self.reprname

    def tzname(self, dt):
        if self.dst(dt):
            return self.dstname
        else:
            return self.stdname

    def utcoffset(self, dt):
        return self.stdoffset + self.dst(dt)

    def dst(self, dt):
        if dt is None or dt.tzinfo is None:
            # An exception may be sensible here, in one or both cases.
            # It depends on how you want to treat them.  The default
            # fromutc() implementation (called by the default astimezone()
            # implementation) passes a datetime with dt.tzinfo is self.
            return timedelta(0)
        assert dt.tzinfo is self

	# In CEST, DST starts at 1am (UTC time) on the last Sunday in March,
	# this is the first Sunday after march 25.
	DSTSTART = datetime(1, 3, 25, 1)
	#and ends at 2am (DST time, 1am UTC time) on the last Sunday of Oct.
	# which is the first Sunday on or after Oct 25.
	DSTEND = datetime(1, 10, 25, 2)
	# Find last Sunday in April & the last in October.
        start = first_sunday_on_or_after(DSTSTART.replace(year=dt.year))
	end = first_sunday_on_or_after(DSTEND.replace(year=dt.year))

        # Can't compare naive to aware objects, so strip the timezone from
        # dt first.
        if start <= dt.replace(tzinfo=None) < end:
            return timedelta(hours=1)
        else:
            return timedelta(0)

# Fout weergeven 'the python way' 
# exception ongeldige klas

class OngeldigeKlas(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)

# hoofdclass...

class Uurrooster:
	def __init__(self, klas):
		self.klas = klas
		self.lessen = []	
		self.timezone = CEST()

	def firstschoolday(self):
		firstseptember = date.today().replace(month=9, day=1)
		# als 1sept dit jaar reeds voorbij is, begon dit academiejaar
		# drie weken na 1 sept vorig kalenderjaar.
		if date.today() < firstseptember:
			firstseptember = firstseptember.replace(year = firstseptember.year - 1)	
		return firstseptember + timedelta( days = 21 - firstseptember.weekday() )	
	
	def retrieve(self):
		klas = re.match("(\d)([a-z]+)(\d?)",self.klas.lower()).groups()
		
		self.jaar = int(klas[0])
		self.richting = klas[1]
		self.groep = int(klas[2])
		lestypes = { 	"t": "theorieles", 
				"o": "oefenzitting", 
				"l": "labo", 
				"u": "uitvoeringstechnieken" }

		
		getfields = { 
		'klasg':	['1pbei','2pbei','3ged','3gei','3get', '1pbem','2pbem', '3gmk','3gmm','3gmo','3gmk', '1pbmp','2pbmp','3gp','1pbat','2pbat','3ga'],
		'klask':	['1ba','1sp','2ba','5las'] ,
		'klas3': 	['3ee','3ei','3em','3cp','3cb','3b'],
		'klas4':	['4ee','4ei','4emae','4emm','4eta','4ete','4cp','4cb','4bb','4bl','4blb']
		}
		
		for getv, klassen in getfields.iteritems():
			if str(self.jaar) + self.richting in klassen:
				getvar = getv
				
		try:
			if getvar:
				pass
		except:
			# FIXME: klas niet geldig: throw exeption!
			# return 
			raise OngeldigeKlas, self.klas + " kan niet worden verwerkt"
			
		http_get = { 	"groep": "1",
				"week" : "1",
				getvar : str(self.jaar) + self.richting }

		url = 'http://uurrooster.denayer.wenk.be/cgi-bin/gjscript?%s' % urllib.urlencode(http_get)
		#htmldata binnenhalen
		try:
			remotefile = urllib.urlopen(url)
		except IOError, (errno, strerror):
			print "Verbindingsfout het ophalen van pagina(%s): %s" % (errno, strerror)
		html = remotefile.read()
		remotefile.close()
		# reglexen compilen, sneller werken?
		# blokken js met info.
		re_blockpatern = re.compile(r"(dag =.*?)les.push",re.DOTALL)
		# strings in javascript array
		re_doublequoted = re.compile(r"\"(.*)\"")
		
		# reglexen voor informatie van het vak.
		re_weekdag = re.compile(r"dag = (\d+);")		# "dag = 1;"
		re_type = re.compile(r"type = '([a-z])';")		# "type = 't';"
		re_begin = re.compile(r"begin = (\d+);")		# "begin = 20;"
		re_einde = re.compile(r"einde = (\d+);")		# "einde = 24;"
		re_weken = re.compile(r"w=new Array\((.*)\);")		# "w=new Array(0, <knip>);"
		re_lokaal = re.compile(r"lok = \(\"(.*)\"\);")		# "lok = (\"A014\");"
		re_naam = re.compile(r"vak=\"(.*)\";")			# "vak = \"datacommunicatie\";"
		re_docenten = re.compile(r"doc = new Array\((.*)\);")	# "doc = new Array(\"MRO\");" 
		re_groepen = re.compile(r"grp = new Array\((.*)\);")	# "grp = new Array(1,1,1,1,1,1,1,1,1,1,1,1);"
		
		data = []
		
		for block in re.findall(re_blockpatern, html):
			weekdag = 	re_weekdag.search(block).group(1)
			type = 		lestypes[re_type.search(block).group(1)]
			begin =		re_begin.search(block).group(1)
			einde =		re_einde.search(block).group(1)
			# verander begin, eindtijd in python time object, maak tijdobject tijdzone-bewust!!!
			begin = time(int(begin) / 2, int(begin) % 2 * 30, tzinfo = self.timezone)
			einde =	time(int(einde) / 2, int(einde) % 2 * 30, tzinfo = self.timezone)
			lokaal =	re_lokaal.search(block).group(1)
			naam =		re_naam.search(block).group(1)
			docenten =	re.findall(re_doublequoted, re_docenten.search(block).group(1))
			b_weken =	re_weken.search(block).group(1).split(",")
			b_groepen =	re_groepen.search(block).group(1).split(",")
			groepen = []
			weken = []
			
			# arrays met 1en en 0en omzetten naar arrays met de indexen
			# van de enen in de arrays
			for n in range(len(b_weken)):
				if b_weken[n] == "1":
					#OPGELET: geen +1 omdat de cgi [1] steeds nul geeft
					weken.append(n)
			
			for n in range(len(b_groepen)):
				if b_groepen[n] == "1":
					groepen.append(n + 1)
			tup = {
			"weekdag":	weekdag,
			"type":		type,	
			"begin":	begin,
			"einde":	einde,
			"weken":	weken,
			"lokaal":	lokaal,
			"naam":		naam,	
			"docenten":	docenten,
			"groepen":	groepen,
			"rawweken":	b_weken
			}
			data.append(tup)
		cursussen = []
		
		for cursus in data:
			if cursus["groepen"].count(self.groep):
				cursussen.append(cursus)
		# self.lessen formaat:
		# week:		schoolweeknummer
		# datum:	datum in python date
		# naam:		naam vak
		# begin:	aanvangtijd die dag python time
		# einde: 	eindtijd...
		# lokaal:	lokaal
		# docenten:	docenten
		# groepen:	groepen die deelnemen
		firstschoolday = self.firstschoolday()
		self.lessen = []
		for cursus in cursussen:
			for week in cursus["weken"]:
				nles = cursus.copy()	#kopie nemen, nles = cursus zou verwijzen naar hetzelfe object
				
				nles['week'] = week
				# weeks = week -1 --> in week 1 moeten 0 weken worden bijgetelt.
				nles['datum'] = firstschoolday + timedelta(days = (int(cursus['weekdag']) - 1), weeks = int(week) - 1)
				#datum bij begin en einde toevoegen zodat deze
				#naar correcte UCT kunnen geconverteerd worden.
				nles['begin'] = datetime.combine(nles['datum'],nles['begin'])
				nles['einde'] = datetime.combine(nles['datum'],nles['einde'])
				self.lessen.append(nles)

	def cursussen(self):
		c = {}
		for les in self.lessen:
			if c.keys().count(les["naam"]) < 1:
				c[les["naam"]] = 1
			else:
				c[les["naam"]] = c[les["naam"]] + 1
		return c
